@@ -16,6 +16,10 @@
  * Visual activation TODO: speed
  * Copyright (c) 2000 Rik Hemsley <rik@kde.org>
  *
+ * White cursor support
+ * TODO: give user the option to choose a certain cursor font
+ * -> Theming
+ *
  * General/Advanced tabs
  * Copyright (c) 2000 Brad Hughes <bhughes@trolltech.com>
  *
@@ -37,18 +41,9 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
-#include <sys/types.h>
-#include <unistd.h>
-#include <sys/stat.h>
-#include <stdlib.h>
-
-#include <qfileinfo.h>
-#include <qstring.h>
-#include <qbuttongroup.h>
-#include <qmessagebox.h>
 #include <qlayout.h>
 #include <qcheckbox.h>
 #undef Below
@@ -56,16 +51,11 @@
 #include <qslider.h>
 #include <qwhatsthis.h>
 #include <qtabwidget.h>
-#include <qpainter.h>
 
 #include <klocale.h>
 #include <kdialog.h>
 #include <kconfig.h>
-#include <kapp.h>
-#include <kstddirs.h>
-#include <kio/netaccess.h>
-#include <kmessagebox.h>
-#include <kalphapainter.h>
+#include <kstandarddirs.h>
 
 #include "mouse.h"
 
@@ -85,8 +75,8 @@ MouseConfig::MouseConfig (QWidget * parent, const char *name)
 
     tabwidget = new QTabWidget(this);
     top->addWidget(tabwidget);
-	
-	tab1 = new KMouseDlg(this);
+
+    tab1 = new KMouseDlg(this);
     QButtonGroup *group = new QButtonGroup( tab1 );
     group->setExclusive( true );
     group->hide();
@@ -116,9 +106,7 @@ MouseConfig::MouseConfig (QWidget * parent, const char *name)
          " check this option.");
     QWhatsThis::add( tab1->doubleClick, wtstr );
 
-    wtstr = i18n("Activates and opens a file or folder with a single click."
-          "When this option is checked, the shape of the mouse pointer"
-         " changes whenever it is over an icon.");
+    wtstr = i18n("Activates and opens a file or folder with a single click.");
     QWhatsThis::add( tab1->singleClick, wtstr );
 
 
@@ -144,11 +132,6 @@ MouseConfig::MouseConfig (QWidget * parent, const char *name)
     wtstr = i18n("Show feedback when clicking an icon");
     QWhatsThis::add( tab1->cbVisualActivate, wtstr );
 
-    connect(tab1->cbLargeCursor, SIGNAL(clicked()), this, SLOT(changed()));
-
-    wtstr = i18n("Use high-visibility large cursor");
-    QWhatsThis::add( tab1->cbLargeCursor, wtstr );
-
     connect(tab1->slAutoSelect, SIGNAL(valueChanged(int)), this, SLOT(changed()));
     connect(tab1->cbVisualActivate, SIGNAL(clicked()), this, SLOT(changed()));
 
@@ -160,21 +143,40 @@ MouseConfig::MouseConfig (QWidget * parent, const char *name)
     connect( tab1->doubleClick, SIGNAL( clicked() ), this, SLOT( slotClick() ) );
     connect( tab1->cbAutoSelect, SIGNAL( clicked() ), this, SLOT( slotClick() ) );
 
+    // Only allow setting reversing scroll polarity if we have scroll buttons
+    unsigned char map[5];
+    if ( XGetPointerMapping(kapp->getDisplay(), map, 5) == 5 )
+    {
+      tab1->cbScrollPolarity->setEnabled( true );
+      tab1->cbScrollPolarity->show();
+    }
+    else
+    {
+      tab1->cbScrollPolarity->setEnabled( false );
+      tab1->cbScrollPolarity->hide();
+    }
+    connect(tab1->cbScrollPolarity, SIGNAL(clicked()), this, SLOT(changed()));
+    connect(tab1->cbScrollPolarity, SIGNAL(clicked()), this, SLOT(slotScrollPolarityChanged()));
+
+    // Cursor theme tab
+    themetab = new ThemePage(this);
+    connect(themetab, SIGNAL(changed(bool)), SLOT(changed()));
+    tabwidget->addTab(themetab, i18n("&Cursor Theme"));
 
     // Advanced tab
     tab2 = new QWidget(0, "Advanced Tab");
-    tabwidget->addTab(tab2, i18n("&Advanced"));
+    tabwidget->addTab(tab2, i18n("Advanced"));
 
     QBoxLayout *lay = new QVBoxLayout(tab2, KDialog::marginHint(),
               KDialog::spacingHint());
 
-    accel = new KIntNumInput(20, tab2);
-    accel->setLabel(i18n("Pointer Acceleration"));
-    accel->setRange(1,20,2);
+    accel = new KDoubleNumInput(2, tab2);
+    accel->setLabel(i18n("Pointer acceleration:"));
+    accel->setPrecision(1);
+    accel->setRange(1,20);
     accel->setSuffix("x");
-    accel->setSteps(1,1);
     lay->addWidget(accel);
-    connect(accel, SIGNAL(valueChanged(int)), this, SLOT(changed()));
+    connect(accel, SIGNAL(valueChanged(double)), this, SLOT(changed()));
 
     wtstr = i18n("This option allows you to change the relationship"
          " between the distance that the mouse pointer moves on the"
@@ -185,16 +187,13 @@ MouseConfig::MouseConfig (QWidget * parent, const char *name)
          " movements of the mouse pointer on the screen even when"
          " you only make a small movement with the physical device."
          " Selecting very high values may result in the mouse pointer"
-         " flying across the screen, making it hard to control!<p>"
-         " You can set the acceleration value by dragging the slider"
-         " button or by clicking the up/down arrows on the spin-button"
-         " to the left of the slider.");
+         " flying across the screen, making it hard to control!");
     QWhatsThis::add( accel, wtstr );
 
     thresh = new KIntNumInput(accel, 20, tab2);
-    thresh->setLabel(i18n("Pointer Threshold"));
-    thresh->setRange(1,20,2);
-    thresh->setSuffix(i18n("pixels"));
+    thresh->setLabel(i18n("Pointer threshold:"));
+    thresh->setRange(0,20,2);
+    thresh->setSuffix(i18n(" pixels"));
     thresh->setSteps(1,1);
     lay->addWidget(thresh);
     connect(thresh, SIGNAL(valueChanged(int)), this, SLOT(changed()));
@@ -207,18 +206,15 @@ MouseConfig::MouseConfig (QWidget * parent, const char *name)
          " there is no acceleration at all, giving you a greater degree"
          " of control over the mouse pointer. With larger movements of"
          " the physical device, you can move the mouse pointer"
-         " rapidly to different areas on the screen.<p>"
-         " You can set the threshold value by dragging the slider button"
-         " or by clicking the up/down arrows on the spin-button to the"
-         " left of the slider.");
+         " rapidly to different areas on the screen.");
     QWhatsThis::add( thresh, wtstr );
 
     // It would be nice if the user had a test field.
     // Selecting such values in milliseconds is not intuitive
     doubleClickInterval = new KIntNumInput(thresh, 2000, tab2);
-    doubleClickInterval->setLabel(i18n("Double Click Interval"));
+    doubleClickInterval->setLabel(i18n("Double click interval:"));
     doubleClickInterval->setRange(0, 2000, 100);
-    doubleClickInterval->setSuffix(i18n("ms"));
+    doubleClickInterval->setSuffix(i18n(" msec"));
     doubleClickInterval->setSteps(100, 100);
     lay->addWidget(doubleClickInterval);
     connect(doubleClickInterval, SIGNAL(valueChanged(int)), this, SLOT(changed()));
@@ -231,12 +227,13 @@ MouseConfig::MouseConfig (QWidget * parent, const char *name)
          " separate clicks.");
     QWhatsThis::add( doubleClickInterval, wtstr );
 
-    dragStartTime = new KIntNumInput(doubleClickInterval, 2000, tab2);
-    dragStartTime->setLabel(i18n("Drag Start Time"));
-    dragStartTime->setRange(0, 2000, 100);
-    dragStartTime->setSuffix(i18n("ms"));
-    dragStartTime->setSteps(100, 100);
     lay->addSpacing(15);
+
+    dragStartTime = new KIntNumInput(doubleClickInterval, 2000, tab2);
+    dragStartTime->setLabel(i18n("Drag start time:"));
+    dragStartTime->setRange(0, 2000, 100);
+    dragStartTime->setSuffix(i18n(" msec"));
+    dragStartTime->setSteps(100, 100);
     lay->addWidget(dragStartTime);
     connect(dragStartTime, SIGNAL(valueChanged(int)), this, SLOT(changed()));
 
@@ -246,9 +243,9 @@ MouseConfig::MouseConfig (QWidget * parent, const char *name)
     QWhatsThis::add( dragStartTime, wtstr );
 
     dragStartDist = new KIntNumInput(dragStartTime, 20, tab2);
-    dragStartDist->setLabel(i18n("Drag Start Distance"));
+    dragStartDist->setLabel(i18n("Drag start distance:"));
     dragStartDist->setRange(1, 20, 2);
-    dragStartDist->setSuffix(i18n("pixels"));
+    dragStartDist->setSuffix(i18n(" pixels"));
     dragStartDist->setSteps(1,1);
     lay->addWidget(dragStartDist);
     connect(dragStartDist, SIGNAL(valueChanged(int)), this, SLOT(changed()));
@@ -259,7 +256,7 @@ MouseConfig::MouseConfig (QWidget * parent, const char *name)
     QWhatsThis::add( dragStartDist, wtstr);
 
     wheelScrollLines = new KIntNumInput(dragStartDist, 3, tab2);
-    wheelScrollLines->setLabel(i18n("Mouse Wheel Scrolls By"));
+    wheelScrollLines->setLabel(i18n("Mouse wheel scrolls by:"));
     wheelScrollLines->setRange(1, 12, 2);
     wheelScrollLines->setSuffix(i18n(" lines"));
     wheelScrollLines->setSteps(1,1);
@@ -268,26 +265,102 @@ MouseConfig::MouseConfig (QWidget * parent, const char *name)
 
     wtstr = i18n("If you use the wheel of a mouse, this value determines the number of lines to scroll for each wheel movement. Note that if this number exceeds the number of visible lines, it will be ignored and the wheel movement will be handled as a page up/down movement.");
     QWhatsThis::add( wheelScrollLines, wtstr);
+    lay->addStretch();
+
+{
+  QWidget *mouse = new QWidget(this, "Mouse Navigation");
+  tabwidget->addTab(mouse, i18n("Mouse Navigation"));
+
+
+  QBoxLayout *vbox = new QVBoxLayout(mouse, KDialog::marginHint(),
+    KDialog::spacingHint());
+
+  QGroupBox *grp = new QGroupBox(i18n("Mouse Navigation"), mouse);
+  grp->setColumnLayout( 0, Qt::Horizontal );
+  vbox->addWidget(grp);
+
+  QVBoxLayout *vvbox = new QVBoxLayout(grp->layout(), KDialog::spacingHint());
+
+  mouseKeys = new QCheckBox(i18n("&Move pointer with keyboard (using the num pad)"), grp);
+  vvbox->addWidget(mouseKeys);
+
+  QBoxLayout *hbox = new QHBoxLayout(vvbox, KDialog::spacingHint());
+  hbox->addSpacing(24);
+  mk_delay = new KIntNumInput(grp);
+  mk_delay->setLabel(i18n("&Acceleration delay:"), AlignVCenter);
+  mk_delay->setSuffix(i18n(" msec"));
+  mk_delay->setRange(1, 1000, 50);
+  hbox->addWidget(mk_delay);
+
+  hbox = new QHBoxLayout(vvbox, KDialog::spacingHint());
+  hbox->addSpacing(24);
+  mk_interval = new KIntNumInput(mk_delay, 0, grp);
+  mk_interval->setLabel(i18n("R&epeat interval:"), AlignVCenter);
+  mk_interval->setSuffix(i18n(" msec"));
+  mk_interval->setRange(1, 1000, 10);
+  hbox->addWidget(mk_interval);
+
+  hbox = new QHBoxLayout(vvbox, KDialog::spacingHint());
+  hbox->addSpacing(24);
+  mk_time_to_max = new KIntNumInput(mk_interval, 0, grp);
+  mk_time_to_max->setLabel(i18n("Acceleration &time:"), AlignVCenter);
+  mk_time_to_max->setRange(1, 5000, 250);
+  hbox->addWidget(mk_time_to_max);
+
+  hbox = new QHBoxLayout(vvbox, KDialog::spacingHint());
+  hbox->addSpacing(24);
+  mk_max_speed = new KIntNumInput(mk_time_to_max, 0, grp);
+  mk_max_speed->setLabel(i18n("Ma&ximum speed:"), AlignVCenter);
+  mk_max_speed->setRange(1, 1000, 10);
+  hbox->addWidget(mk_max_speed);
+
+  hbox = new QHBoxLayout(vvbox, KDialog::spacingHint());
+  hbox->addSpacing(24);
+  mk_curve = new KIntNumInput(mk_max_speed, 0, grp);
+  mk_curve->setLabel(i18n("Acceleration &profile:"), AlignVCenter);
+  mk_curve->setRange(-1000, 1000, 100);
+  hbox->addWidget(mk_curve);
+
+  connect(mouseKeys, SIGNAL(clicked()), this, SLOT(checkAccess()));
+  connect(mouseKeys, SIGNAL(clicked()), this, SLOT(changed()));
+  connect(mk_delay, SIGNAL(valueChanged(int)), this, SLOT(changed()));
+  connect(mk_interval, SIGNAL(valueChanged(int)), this, SLOT(changed()));
+  connect(mk_time_to_max, SIGNAL(valueChanged(int)), this, SLOT(changed()));
+  connect(mk_max_speed, SIGNAL(valueChanged(int)), this, SLOT(changed()));
+  connect(mk_curve, SIGNAL(valueChanged(int)), this, SLOT(changed()));
+
+  vbox->addStretch();
+}
 
-    lay->addStretch(1);
 
     config = new KConfig("kcminputrc");
     settings = new MouseSettings;
     load();
 }
 
+
+void MouseConfig::checkAccess()
+{
+  mk_delay->setEnabled(mouseKeys->isChecked());
+  mk_interval->setEnabled(mouseKeys->isChecked());
+  mk_time_to_max->setEnabled(mouseKeys->isChecked());
+  mk_max_speed->setEnabled(mouseKeys->isChecked());
+  mk_curve->setEnabled(mouseKeys->isChecked());
+}
+
+
 MouseConfig::~MouseConfig()
 {
     delete config;
     delete settings;
 }
 
-int MouseConfig::getAccel()
+double MouseConfig::getAccel()
 {
   return accel->value();
 }
 
-void MouseConfig::setAccel(int val)
+void MouseConfig::setAccel(double val)
 {
   accel->setValue(val);
 }
@@ -331,6 +404,9 @@ void MouseConfig::load()
 
   tab1->rightHanded->setEnabled(settings->handedEnabled);
   tab1->leftHanded->setEnabled(settings->handedEnabled);
+  if ( tab1->cbScrollPolarity->isEnabled() )
+    tab1->cbScrollPolarity->setEnabled(settings->handedEnabled);
+  tab1->cbScrollPolarity->setChecked( settings->reverseScrollPolarity );
 
   setAccel(settings->accelRate);
   setThreshold(settings->thresholdMove);
@@ -341,7 +417,7 @@ void MouseConfig::load()
   dragStartDist->setValue(settings->dragStartDist);
   wheelScrollLines->setValue(settings->wheelScrollLines);
 
-  tab1->singleClick->setChecked( settings->changeCursor );
+  tab1->singleClick->setChecked( settings->singleClick );
   tab1->doubleClick->setChecked(!settings->singleClick);
   tab1->cb_pointershape->setChecked(settings->changeCursor);
   tab1->cbAutoSelect->setChecked( settings->autoSelectDelay >= 0 );
@@ -350,8 +426,23 @@ void MouseConfig::load()
   else
      tab1->slAutoSelect->setValue( settings->autoSelectDelay );
   tab1->cbVisualActivate->setChecked( settings->visualActivate );
-  tab1->cbLargeCursor->setChecked( settings->largeCursor );
   slotClick();
+
+
+  KConfig ac("kaccessrc", true);
+
+  ac.setGroup("Mouse");
+  mouseKeys->setChecked(ac.readBoolEntry("MouseKeys", false));
+  mk_delay->setValue(ac.readNumEntry("MKDelay", 160));
+  mk_interval->setValue(ac.readNumEntry("MKInterval", 5));
+  mk_time_to_max->setValue(ac.readNumEntry("MKTimeToMax", 1000));
+  mk_max_speed->setValue(ac.readNumEntry("MKMaxSpeed", 500));
+  mk_curve->setValue(ac.readNumEntry("MKCurve", 0));
+
+  themetab->load();
+
+  checkAccess();
+  changed();
 }
 
 void MouseConfig::save()
@@ -369,18 +460,31 @@ void MouseConfig::save()
   settings->visualActivate = tab1->cbVisualActivate->isChecked();
 //  settings->changeCursor = tab1->singleClick->isChecked();
   settings->changeCursor = tab1->cb_pointershape->isChecked();
-  settings->largeCursor = tab1->cbLargeCursor->isChecked();
-
-  // Check if the user has asked us not to remind them that KDE needs
-  // restarting after a cursor size change.
-  bool wasLargeCursor =
-      config->readBoolEntry("LargeCursor", KDE_DEFAULT_LARGE_CURSOR);
+  settings->reverseScrollPolarity = tab1->cbScrollPolarity->isChecked();
 
   settings->apply();
   settings->save(config);
-  if (settings->largeCursor != wasLargeCursor) {
-    KMessageBox::information(this, i18n("KDE must be restarted for the cursor size change to take effect"), QString::null, "DoNotRemindCursor");
-  }
+
+  KConfig ac("kaccessrc", false);
+
+  ac.setGroup("Mouse");
+
+  ac.writeEntry("MouseKeys", mouseKeys->isChecked());
+  ac.writeEntry("MKDelay", mk_delay->value());
+  ac.writeEntry("MKInterval", mk_interval->value());
+  ac.writeEntry("MKTimeToMax", mk_time_to_max->value());
+  ac.writeEntry("MKMaxSpeed", mk_max_speed->value());
+  ac.writeEntry("MKCurve", mk_curve->value());
+
+  config->sync();
+
+  themetab->save();
+
+  // restart kaccess
+  kapp->startServiceByDesktopName("kaccess");
+
+  KCModule::changed(false);
+
 }
 
 void MouseConfig::defaults()
@@ -388,6 +492,7 @@ void MouseConfig::defaults()
     setThreshold(2);
     setAccel(2);
     setHandedness(RIGHT_HANDED);
+    tab1->cbScrollPolarity->setChecked( false );
     doubleClickInterval->setValue(400);
     dragStartTime->setValue(500);
     dragStartDist->setValue(4);
@@ -396,8 +501,20 @@ void MouseConfig::defaults()
     tab1->cbAutoSelect->setChecked( KDE_DEFAULT_AUTOSELECTDELAY != -1 );
     tab1->slAutoSelect->setValue( KDE_DEFAULT_AUTOSELECTDELAY == -1 ? 50 : KDE_DEFAULT_AUTOSELECTDELAY );
     tab1->singleClick->setChecked( KDE_DEFAULT_SINGLECLICK );
-    tab1->cbLargeCursor->setChecked( KDE_DEFAULT_LARGE_CURSOR );
+    tab1->cbVisualActivate->setChecked( KDE_DEFAULT_VISUAL_ACTIVATE );
+    tab1->cb_pointershape->setChecked(KDE_DEFAULT_CHANGECURSOR);
     slotClick();
+
+  mouseKeys->setChecked(false);
+  mk_delay->setValue(160);
+  mk_interval->setValue(5);
+  mk_time_to_max->setValue(1000);
+  mk_max_speed->setValue(500);
+  mk_curve->setValue(0);
+
+  checkAccess();
+
+  changed();
 }
 
 
@@ -425,26 +542,29 @@ void MouseConfig::slotClick()
 
 void MouseConfig::changed()
 {
-
   emit KCModule::changed(true);
 }
+
 /** No descriptions */
 void MouseConfig::slotHandedChanged(int val){
-  if(val==0)
+  if(val==RIGHT_HANDED)
     tab1->mousePix->setPixmap(locate("data", "kcminput/pics/mouse_rh.png"));
   else
     tab1->mousePix->setPixmap(locate("data", "kcminput/pics/mouse_lh.png"));
+  settings->m_handedNeedsApply = true;
 }
 
 void MouseSettings::load(KConfig *config)
 {
   int accel_num, accel_den, threshold;
+  double accel;
   XGetPointerControl( kapp->getDisplay(),
               &accel_num, &accel_den, &threshold );
-  accel_num /= accel_den;   // integer acceleration only
+  accel = float(accel_num) / float(accel_den);
 
   // get settings from X server
   int h = RIGHT_HANDED;
+  bool revScroll = false;
   unsigned char map[5];
   num_buttons = XGetPointerMapping(kapp->getDisplay(), map, 5);
 
@@ -484,11 +604,14 @@ void MouseSettings::load(KConfig *config)
       handedEnabled = false;
       break;
     }
+  if ( handedEnabled && num_buttons == 5 && ( int )map[3] == 5 && ( int )map[4] == 4 )
+    revScroll = true;
+
 
   config->setGroup("Mouse");
-  int a = config->readNumEntry("Acceleration",-1);
+  double a = config->readDoubleNumEntry("Acceleration",-1);
   if (a == -1)
-    accelRate = accel_num;
+    accelRate = accel;
   else
     accelRate = a;
 
@@ -505,6 +628,8 @@ void MouseSettings::load(KConfig *config)
     handed = LEFT_HANDED;
   else if (key == NULL)
     handed = h;
+  reverseScrollPolarity = config->readBoolEntry( "ReverseScrollPolarity", false );
+  m_handedNeedsApply = ( (handed != h) || (reverseScrollPolarity != revScroll) );
 
   // SC/DC/AutoSelect/ChangeCursor
   config->setGroup("KDE");
@@ -517,18 +642,16 @@ void MouseSettings::load(KConfig *config)
   autoSelectDelay = config->readNumEntry("AutoSelectDelay", KDE_DEFAULT_AUTOSELECTDELAY);
   visualActivate = config->readBoolEntry("VisualActivate", KDE_DEFAULT_VISUAL_ACTIVATE);
   changeCursor = config->readBoolEntry("ChangeCursor", KDE_DEFAULT_CHANGECURSOR);
-  largeCursor = config->readBoolEntry("LargeCursor", KDE_DEFAULT_LARGE_CURSOR);
 }
 
 void MouseSettings::apply()
 {
   XChangePointerControl( kapp->getDisplay(),
-                         true, true, accelRate, 1, thresholdMove);
-
+                         true, true, int(qRound(accelRate*10)), 10, thresholdMove);
 
   unsigned char map[5];
   int remap=1;
-  if (handedEnabled) {
+  if (handedEnabled && m_handedNeedsApply) {
       switch (num_buttons) {
       case 1:
           map[0] = (unsigned char) 1;
@@ -557,20 +680,20 @@ void MouseSettings::apply()
           break;
       case 5:
           // Intellimouse case, where buttons 1-3 are left, middle, and
-          // right, and 4-5 are up/down
+          // right, and 4-5 are up/down depending on scroll polarity
           if (handed == RIGHT_HANDED) {
               map[0] = (unsigned char) 1;
               map[1] = (unsigned char) 2;
               map[2] = (unsigned char) 3;
-              map[3] = (unsigned char) 4;
-              map[4] = (unsigned char) 5;
+              map[3] = reverseScrollPolarity ? (unsigned char) 5 : (unsigned char) 4;
+              map[4] = reverseScrollPolarity ? (unsigned char) 4 : (unsigned char) 5;
           }
           else {
               map[0] = (unsigned char) 3;
               map[1] = (unsigned char) 2;
               map[2] = (unsigned char) 1;
-              map[3] = (unsigned char) 4;
-              map[4] = (unsigned char) 5;
+              map[3] = reverseScrollPolarity ? (unsigned char) 5 : (unsigned char) 4;
+              map[4] = reverseScrollPolarity ? (unsigned char) 4 : (unsigned char) 5;
           }
           break;
       default: {
@@ -587,28 +710,9 @@ void MouseSettings::apply()
                                             num_buttons)) == MappingBusy)
               /* keep trying until the pointer is free */
           { };
+      m_handedNeedsApply = false;
   }
 
-  // Make sure we have the 'font' resource dir registered and can find the
-  // override dir.
-  //
-  // Next, if the user wants large cursors, copy the font
-  // cursor_large.pcf.gz to (localkdedir)/share/fonts/override/cursor.pcf.gz.
-  // Else remove the font cursor.pcf.gz from (localkdedir)/share/fonts/override.
-  //
-  // Run mkfontdir to update fonts.dir in that dir.
-
-  KGlobal::dirs()->addResourceType("font", "share/fonts/");
-  QString overrideDir = locateLocal("font", "override/");
-  QString font = locate("data", "kcminput/cursor_large.pcf.gz");
-  QString installedFont = overrideDir + "/cursor.pcf.gz";
-
-  if (!largeCursor)
-    unlink(QFile::encodeName(installedFont));
-  else if (!!font)
-    KIO::NetAccess::copy(font, installedFont);
-
-  system(QFile::encodeName("mkfontdir " + overrideDir));
 }
 
 void MouseSettings::save(KConfig *config)
@@ -620,6 +724,7 @@ void MouseSettings::save(KConfig *config)
       config->writeEntry("MouseButtonMapping",QString("RightHanded"));
   else
       config->writeEntry("MouseButtonMapping",QString("LeftHanded"));
+  config->writeEntry( "ReverseScrollPolarity", reverseScrollPolarity );
 
   config->setGroup("KDE");
   config->writeEntry("DoubleClickInterval", doubleClickInterval, true, true);
@@ -629,10 +734,17 @@ void MouseSettings::save(KConfig *config)
   config->writeEntry("SingleClick", singleClick, true, true);
   config->writeEntry("AutoSelectDelay", autoSelectDelay, true, true );
   config->writeEntry("VisualActivate", visualActivate, true, true);
-  config->writeEntry("ChangeCursor", changeCursor, true, true );
-  config->writeEntry("LargeCursor", largeCursor, true, true );
+
+  // the following need to be in the local config file so they are read
+  // by kcminit (which only reads local config for speed up)
+  config->writeEntry("ChangeCursor", changeCursor );
   config->sync();
   KIPC::sendMessageAll(KIPC::SettingsChanged, KApplication::SETTINGS_MOUSE);
 }
 
+void MouseConfig::slotScrollPolarityChanged()
+{
+  settings->m_handedNeedsApply = true;
+}
+
 #include "mouse.moc"
