@@ -1,10 +1,13 @@
 /* $TOG: xdmcp.c /main/18 1998/06/04 11:50:41 barstow $ */
-/* $Id: xdmcp.c,v 1.13.2.1 2001/09/23 11:29:56 ossi Exp $ */
 /*
 
 Copyright 1988, 1998  The Open Group
 
-All Rights Reserved.
+Permission to use, copy, modify, distribute, and sell this software and its
+documentation for any purpose is hereby granted without fee, provided that
+the above copyright notice appear in all copies and that both that
+copyright notice and this permission notice appear in supporting
+documentation.
 
 The above copyright notice and this permission notice shall be included
 in all copies or substantial portions of the Software.
@@ -33,11 +36,11 @@ from The Open Group.
  */
 
 #include "dm.h"
+#include "dm_error.h"
 
 #ifdef XDMCP
 
 #include "dm_auth.h"
-#include "dm_error.h"
 #include "dm_socket.h"
 
 #include <X11/X.h>
@@ -46,46 +49,25 @@ from The Open Group.
 #include <sys/types.h>
 #include <ctype.h>
 
-#ifndef MINIX
-# ifndef X_NO_SYS_UN
-#  ifndef Lynx
-#   include <sys/un.h>
-#  else
-#   include <un.h>
-#  endif
+#ifndef X_NO_SYS_UN
+# ifndef Lynx
+#  include <sys/un.h>
+# else
+#  include <un.h>
 # endif
-# include <netdb.h>
-#else /* MINIX */
-# include <net/hton.h>
-# include <net/gen/netdb.h>
-#endif /* !MINIX */
-
-#ifdef X_NOT_STDC_ENV
-# define Time_t long
-extern Time_t time ();
-#else
-# include <time.h>
-# define Time_t time_t
 #endif
-
-#ifdef MINIX
-struct sockaddr_un
-{
-	u16_t   sun_family;
-	char    sun_path[62];
-};
-static char read_buffer[XDM_MAX_MSGLEN+sizeof(udp_io_hdr_t)];
-static int read_inprogress;
-static int read_size;
-# define select(n,r,w,x,t) nbio_select(n,r,w,x,t)
+#if defined(__SVR4) && defined(__sun)
+/*
+ * make sure we get the resolver's version of gethostbyname
+ * otherwise we may not get all the addresses!
+ */
+# define gethostbyname res_gethostbyname
 #endif
-
-#define getString(name,len)	((name = malloc (len + 1)) ? 1 : 0)
+#include <netdb.h>
 
 /*
  * misc externs
  */
-extern int Rescan, ChildReady;
 extern int sourceAddress;
 
 /*
@@ -99,41 +81,32 @@ static void request_respond (struct sockaddr *from, int fromlen, int length);
 static void send_accept (struct sockaddr *to, int tolen, CARD32 sessionID, ARRAY8Ptr authenticationName, ARRAY8Ptr authenticationData, ARRAY8Ptr authorizationName, ARRAY8Ptr authorizationData);
 static void send_alive (struct sockaddr *from, int fromlen, int length);
 static void send_decline (struct sockaddr *to, int tolen, ARRAY8Ptr authenticationName, ARRAY8Ptr authenticationData, ARRAY8Ptr status);
-static void send_failed (struct sockaddr *from, int fromlen, char *name, CARD32 sessionID, char *reason);
+static void send_failed (struct sockaddr *from, int fromlen, const char *name, CARD32 sessionID, const char *reason);
 static void send_refuse (struct sockaddr *from, int fromlen, CARD32 sessionID);
 static void send_unwilling (struct sockaddr *from, int fromlen, ARRAY8Ptr authenticationName, ARRAY8Ptr status);
 static void send_willing (struct sockaddr *from, int fromlen, ARRAY8Ptr authenticationName, ARRAY8Ptr status);
 
 
 int	xdmcpFd = -1;
-int	chooserFd = -1;
-
-FD_TYPE	WellKnownSocketsMask;
-int	WellKnownSocketsMax;
-
-#define pS(s)	((s) ? ((char *) (s)) : "empty string")
 
 void
 DestroyWellKnownSockets (void)
 {
     if (xdmcpFd != -1)
     {
-	close (xdmcpFd);
+	CloseNClearCloseOnFork (xdmcpFd);
+	UnregisterInput (xdmcpFd);
 	xdmcpFd = -1;
     }
-    if (chooserFd != -1)
-    {
-	close (chooserFd);
-	chooserFd = -1;
-    }
 }
 
 int
 AnyWellKnownSockets (void)
 {
-    return xdmcpFd != -1 || chooserFd != -1;
+    return xdmcpFd != -1;
 }
 
+
 static XdmcpBuffer	buffer;
 
 /*ARGSUSED*/
@@ -252,7 +225,7 @@ indirect_respond (
     XdmcpHeader	    header;
     int		    localHostAsWell;
     
-    Debug ("Indirect respond %d\n", length);
+    Debug ("<indirect> respond %d\n", length);
     if (!XdmcpReadARRAYofARRAY8 (&buffer, &queryAuthenticationNames))
 	return;
     expectedLen = 1;
@@ -264,18 +237,18 @@ indirect_respond (
 	/*
 	 * set up the forward query packet
 	 */
-    	header.version = XDM_PROTOCOL_VERSION;
-    	header.opcode = (CARD16) FORWARD_QUERY;
-    	header.length = 0;
-    	header.length += 2 + clientAddress.length;
-    	header.length += 2 + clientPort.length;
-    	header.length += 1;
-    	for (i = 0; i < (int)queryAuthenticationNames.length; i++)
+	header.version = XDM_PROTOCOL_VERSION;
+	header.opcode = (CARD16) FORWARD_QUERY;
+	header.length = 0;
+	header.length += 2 + clientAddress.length;
+	header.length += 2 + clientPort.length;
+	header.length += 1;
+	for (i = 0; i < (int)queryAuthenticationNames.length; i++)
 	    header.length += 2 + queryAuthenticationNames.data[i].length;
-    	XdmcpWriteHeader (&buffer, &header);
-    	XdmcpWriteARRAY8 (&buffer, &clientAddress);
-    	XdmcpWriteARRAY8 (&buffer, &clientPort);
-    	XdmcpWriteARRAYofARRAY8 (&buffer, &queryAuthenticationNames);
+	XdmcpWriteHeader (&buffer, &header);
+	XdmcpWriteARRAY8 (&buffer, &clientAddress);
+	XdmcpWriteARRAY8 (&buffer, &clientPort);
+	XdmcpWriteARRAYofARRAY8 (&buffer, &queryAuthenticationNames);
 
 	localHostAsWell = ForEachMatchingIndirectHost (&clientAddress, connectionType, sendForward, (char *) 0);
 	
@@ -287,56 +260,24 @@ indirect_respond (
     }
     else
     {
-	Debug ("Indirect length error got %d expect %d\n", length, expectedLen);
+	Debug ("<indirect> length error got %d expect %d\n", length, expectedLen);
     }
     XdmcpDisposeARRAYofARRAY8 (&queryAuthenticationNames);
 }
 
-static void
+void
 ProcessRequestSocket (void)
 {
     XdmcpHeader		header;
     struct sockaddr	addr;
     int			addrlen = sizeof addr;
-#ifdef MINIX
-    int			r;
-#endif
-
-#ifdef MINIX
-    if (read_inprogress) abort();
-    if (read_size == 0)
-    {
-    	r= read(xdmcpFd, read_buffer, sizeof(read_buffer));
-    	if (r == -1 && errno == EINPROGRESS)
-    	{
-    		read_inprogress= 1;
-    		nbio_inprogress(xdmcpFd, ASIO_READ, 1 /* read */,
-    			0 /* write */, 0 /* exception */);
-    	}
-    	else if (r <= 0)
-    	{
-    		LogError("read error: %s\n",
-    			r == 0 ?  "EOF" : strerror(errno));
-		return;
-	}
-    }
-#endif
 
     Debug ("ProcessRequestSocket\n");
     bzero ((char *) &addr, sizeof (addr));
-#ifdef MINIX
-    if (!MNX_XdmcpFill (xdmcpFd, &buffer, &addr, &addrlen,
-    	read_buffer, read_size))
-    {
-	return;
-    }
-    read_size= 0;
-#else
     if (!XdmcpFill (xdmcpFd, &buffer, (XdmcpNetaddr) &addr, &addrlen)) {
 	Debug ("XdmcpFill failed\n");
 	return;
     }
-#endif
     if (!XdmcpReadHeader (&buffer, &header)) {
 	Debug ("XdmcpReadHeader failed\n");
 	return;
@@ -373,52 +314,6 @@ ProcessRequestSocket (void)
     }
 }
 
-void
-WaitForSomething (void)
-{
-    FD_TYPE	reads;
-    int	nready;
-
-    Debug ("WaitForSomething\n");
-    if (AnyWellKnownSockets () && !ChildReady) {
-	reads = WellKnownSocketsMask;
-#ifdef MINIX__NOT
-	{
-		struct timeval tv;
-		tv.tv_sec= 5;
-		tv.tv_usec= 0;
-		nready = select (WellKnownSocketsMax + 1, &reads, 0, 0, &tv);
-		ChildReady= 1;
-	}
-#else
-# if defined(hpux)
-	nready = select (WellKnownSocketsMax + 1, (int*)reads.fds_bits, 0, 0, 0);
-# else
-	nready = select (WellKnownSocketsMax + 1, &reads, 0, 0, 0);
-# endif
-#endif
-	Debug ("select returns %d.  Rescan: %d  ChildReady: %d\n",
-		nready, Rescan, ChildReady);
-	if (nready > 0)
-	{
-	    if (xdmcpFd >= 0 && FD_ISSET (xdmcpFd, &reads))
-		ProcessRequestSocket ();
-	    if (chooserFd >= 0 && FD_ISSET (chooserFd, &reads))
-#ifdef ISC
-	        if (!ChildReady) {
-	           WaitForSomething ();
-                } else
-#endif
-		ProcessChooserSocket (chooserFd);
-	}
-	if (ChildReady)
-	{
-	    WaitForChild ();
-	}
-    } else
-	WaitForChild ();
-}
-
 /*
  * respond to a request on the UDP socket.
  */
@@ -427,15 +322,13 @@ static ARRAY8	Hostname;
 
 void
 registerHostname (
-    char    *name,
-    int	    namelen)
+    const char	*name,
+    int		namelen)
 {
-    int	i;
-
     if (!XdmcpReallocARRAY8 (&Hostname, namelen))
 	return;
-    for (i = 0; i < namelen; i++)
-	Hostname.data[i] = name[i];
+    memcpy (Hostname.data, name, namelen);
+	
 }
 
 static void
@@ -465,7 +358,7 @@ query_respond (
     int		    fromlen,
     int		    length)
 {
-    Debug ("Query respond %d\n", length);
+    Debug ("<query> respond %d\n", length);
     direct_query_respond (from, fromlen, length, QUERY);
 }
 
@@ -494,22 +387,14 @@ NetworkAddressToName(
 	    CARD8		*data;
 	    struct hostent	*hostent;
 	    char		*name;
-	    char		*localhost;
+	    const char		*localhost;
 	    int			 multiHomed = 0;
 
 	    data = connectionAddress->data;
 	    hostent = gethostbyaddr ((char *)data,
 				     connectionAddress->length, AF_INET);
 	    if (sourceAddress && hostent) {
-#if defined(__SVR4) && defined(__sun)
-		/*
-		 * make sure we get the resolver's version of gethostbyname
-		 * otherwise we may not get all the addresses!
-		 */
-		hostent = (struct hostent *) res_gethostbyname(hostent->h_name);
-#else
 		hostent = gethostbyname(hostent->h_name);
-#endif
 		if (hostent)
 			multiHomed = hostent->h_addr_list[1] != NULL;
 	    }
@@ -524,25 +409,21 @@ NetworkAddressToName(
 			&& !multiHomed)
 	    {
 		if (!strcmp (localhost, hostent->h_name))
-		{
-		    if (!getString (name, 10))
-			return 0;
-		    sprintf (name, ":%d", displayNumber);
-		}
+		    ASPrintf (&name, "localhost:%d", displayNumber);
 		else
 		{
 		    if (removeDomainname)
 		    {
-		    	char    *localDot, *remoteDot;
+			char    *localDot, *remoteDot;
     
 			/* check for a common domain name.  This
 			 * could reduce names by recognising common
 			 * super-domain names as well, but I don't think
 			 * this is as useful, and will confuse more
 			 * people
- 			 */
-		    	if ((localDot = strchr(localhost, '.')) &&
-		            (remoteDot = strchr(hostent->h_name, '.')))
+			 */
+			if ((localDot = strchr(localhost, '.')) &&
+			    (remoteDot = strchr(hostent->h_name, '.')))
 			{
 			    /* smash the name in place; it won't
 			     * be needed later.
@@ -552,20 +433,15 @@ NetworkAddressToName(
 			}
 		    }
 
-		    if (!getString (name, strlen (hostent->h_name) + 10))
-			return 0;
-		    sprintf (name, "%s:%d", hostent->h_name, displayNumber);
+		    ASPrintf (&name, "%s:%d", hostent->h_name, displayNumber);
 		}
 	    }
 	    else
 	    {
-		if (!getString (name, 25))
-		    return 0;
 		if (multiHomed)
 		    data = (CARD8 *) &((struct sockaddr_in *)originalAddress)->
 				sin_addr.s_addr;
-		sprintf(name, "%d.%d.%d.%d:%d",
-			data[0], data[1], data[2], data[3], displayNumber);
+		ASPrintf (&name, "%[4|'.'hhu:%d", data, displayNumber);
 	    }
 	    return name;
 	}
@@ -593,7 +469,7 @@ forward_respond (
     int		    expectedLen;
     int		    i;
     
-    Debug ("Forward respond %d\n", length);
+    Debug ("<forward> respond %d\n", length);
     clientAddress.length = 0;
     clientAddress.data = 0;
     clientPort.length = 0;
@@ -617,19 +493,17 @@ forward_respond (
 	    j = 0;
 	    for (i = 0; i < (int)clientPort.length; i++)
 		j = j * 256 + clientPort.data[i];
-	    Debug ("Forward client address (port %d)", j);
-	    for (i = 0; i < (int)clientAddress.length; i++)
-		Debug (" %d", clientAddress.data[i]);
-	    Debug ("\n");
-    	    switch (from->sa_family)
-    	    {
+	    Debug ("<forward> client address (port %d) %[*hhu\n", j,
+		   clientAddress.length, clientAddress.data);
+	    switch (from->sa_family)
+	    {
 #ifdef AF_INET
 	    case AF_INET:
 		{
 		    struct sockaddr_in	in_addr;
 
 		    if (clientAddress.length != 4 ||
-		        clientPort.length != 2)
+			clientPort.length != 2)
 		    {
 			goto badAddress;
 		    }
@@ -659,7 +533,7 @@ forward_respond (
 		    memmove( un_addr.sun_path, clientAddress.data, clientAddress.length);
 		    un_addr.sun_path[clientAddress.length] = '\0';
 		    client = (struct sockaddr *) &un_addr;
-#if defined(BSD44SOCKETS) && !defined(Lynx)
+#if defined(BSD44SOCKETS) && !defined(Lynx) && defined(UNIXCONN)
 		    un_addr.sun_len = strlen(un_addr.sun_path);
 		    clientlen = SUN_LEN(&un_addr);
 #else
@@ -678,11 +552,11 @@ forward_respond (
 	    case AF_DECnet:
 		goto badAddress;
 #endif
-    	    }
+	    }
 	}
 	else
 	{
-	    Debug ("Forward length error got %d expect %d\n", length, expectedLen);
+	    Debug ("<forward> length error got %d expect %d\n", length, expectedLen);
 	}
     }
 badAddress:
@@ -700,12 +574,10 @@ send_willing (
 {
     XdmcpHeader	header;
 
-    Debug ("Send willing %*.*s %*.*s\n", authenticationName->length,
-					 authenticationName->length,
-					 pS(authenticationName->data),
+    Debug ("send <willing> %.*s %.*s\n", authenticationName->length,
+					 authenticationName->data,
 					 status->length,
-					 status->length,
-					 pS(status->data));
+					 status->data);
     header.version = XDM_PROTOCOL_VERSION;
     header.opcode = (CARD16) WILLING;
     header.length = 6 + authenticationName->length +
@@ -726,12 +598,10 @@ send_unwilling (
 {
     XdmcpHeader	header;
 
-    Debug ("Send unwilling %*.*s %*.*s\n", authenticationName->length,
-					 authenticationName->length,
-					 pS(authenticationName->data),
-					 status->length,
-					 status->length,
-					 pS(status->data));
+    Debug ("send <unwilling> %.*s %.*s\n", authenticationName->length,
+					   authenticationName->data,
+					   status->length,
+					   status->data);
     header.version = XDM_PROTOCOL_VERSION;
     header.opcode = (CARD16) UNWILLING;
     header.length = 4 + Hostname.length + status->length;
@@ -751,9 +621,9 @@ void init_session_id(void)
      * incarnation so we don't say "Alive" to those displays.
      * Start with low digits 0 to make debugging easier.
      */
-    globalSessionID = (time((Time_t)0)&0x7fff) * 16000;
+    globalSessionID = (time((Time_t *)0)&0x7fff) * 16000;
 }
-    
+
 static ARRAY8 outOfMemory = { (CARD16) 13, (CARD8Ptr) "Out of memory" };
 static ARRAY8 noValidAddr = { (CARD16) 16, (CARD8Ptr) "No valid address" };
 static ARRAY8 noValidAuth = { (CARD16) 22, (CARD8Ptr) "No valid authorization" };
@@ -779,7 +649,7 @@ request_respond (
     ARRAY8	    authorizationName, authorizationData;
     ARRAY8Ptr	    connectionAddress;
 
-    Debug ("Request respond %d\n", length);
+    Debug ("<request> respond %d\n", length);
     connectionTypes.data = 0;
     connectionAddresses.data = 0;
     authenticationName.data = 0;
@@ -810,7 +680,7 @@ request_respond (
 	expectlen += 2 + manufacturerDisplayID.length;	/* displayID */
 	if (expectlen != length)
 	{
-	    Debug ("Request length error got %d expect %d\n", length, expectlen);
+	    Debug ("<request> length error got %d expect %d\n", length, expectlen);
 	    goto abort;
 	}
 	if (connectionTypes.length == 0 ||
@@ -885,7 +755,7 @@ request_respond (
 	if (pdpy)
 	{
 	    send_accept (from, fromlen, pdpy->sessionID,
-				        &authenticationName,
+					&authenticationName,
 					&authenticationData,
 					&authorizationName,
 					&authorizationData);
@@ -896,7 +766,7 @@ decline:    ;
 	    send_decline (from, fromlen, &authenticationName,
 				 &authenticationData,
 				 reason);
-            if (pdpy)
+	    if (pdpy)
 		DisposeProtoDisplay (pdpy);
 	}
     }
@@ -921,7 +791,7 @@ send_accept (
 {
     XdmcpHeader	header;
 
-    Debug ("Accept Session ID %ld\n", (long) sessionID);
+    Debug ("<accept> session ID %ld\n", (long) sessionID);
     header.version = XDM_PROTOCOL_VERSION;
     header.opcode = (CARD16) ACCEPT;
     header.length = 4;			    /* session ID */
@@ -948,7 +818,7 @@ send_decline (
 {
     XdmcpHeader	header;
 
-    Debug ("Decline %*.*s\n", status->length, status->length, pS(status->data));
+    Debug ("<decline> %.*s\n", status->length, status->data);
     header.version = XDM_PROTOCOL_VERSION;
     header.opcode = (CARD16) DECLINE;
     header.length = 0;
@@ -980,7 +850,7 @@ manage (
     ARRAY8		clientAddress, clientPort;
     CARD16		connectionType;
 
-    Debug ("Manage %d\n", length);
+    Debug ("<manage> %d\n", length);
     displayClass.data = 0;
     displayClass.length = 0;
     if (XdmcpReadCARD32 (&buffer, &sessionID) &&
@@ -992,11 +862,11 @@ manage (
 		    2 + displayClass.length;	/* displayClass */
 	if (expectlen != length)
 	{
-	    Debug ("Manage length error got %d expect %d\n", length, expectlen);
+	    Debug ("<manage> length error got %d expect %d\n", length, expectlen);
 	    goto abort;
 	}
 	pdpy = FindProtoDisplay ((XdmcpNetaddr) from, fromlen, displayNumber);
-	Debug ("Manage Session ID %ld, pdpy %p\n", (long) sessionID, pdpy);
+	Debug ("<manage> session ID %ld, pdpy %p\n", (long) sessionID, pdpy);
 	if (!pdpy || pdpy->sessionID != sessionID)
 	{
 	    /*
@@ -1013,9 +883,9 @@ manage (
 		     Debug("manage: got duplicate pkt, ignoring\n");
 		     goto abort;
 	    }
-	    Debug ("Session ID %ld refused\n", (long) sessionID);
+	    Debug ("session ID %ld refused\n", (long) sessionID);
 	    if (pdpy)
-		Debug ("Existing Session ID %ld\n", (long) pdpy->sessionID);
+		Debug ("existing session ID %ld\n", (long) pdpy->sessionID);
 	    send_refuse (from, fromlen, sessionID);
 	}
 	else
@@ -1026,19 +896,19 @@ manage (
 					 pdpy->displayNumber);
 	    if (!name)
 	    {
-		Debug ("Could not compute display name\n");
+		Debug ("could not compute display name\n");
 		send_failed (from, fromlen, "(no name)", sessionID, "out of memory");
 		goto abort;
 	    }
-	    Debug ("Computed display name: %s\n", name);
+	    Debug ("computed display name: %s\n", name);
 	    if ((d = FindDisplayByName (name)))
 	    {
-		Debug ("Terminating active session for %s\n", d->name);
+		Debug ("terminating active session for %s\n", d->name);
 		StopDisplay (d);
 	    }
 	    if (displayClass.length)
 	    {
-		if (!StrNDup(&class2, displayClass.data, displayClass.length))
+		if (!StrNDup(&class2, (char *)displayClass.data, displayClass.length))
 		{
 		    send_failed (from, fromlen, name, sessionID, "out of memory");
 		    goto abort;
@@ -1056,9 +926,9 @@ manage (
 		send_failed (from, fromlen, name, sessionID, "out of memory");
 		goto abort;
 	    }
-	    d->displayType = Foreign | Transient | FromXDMCP;
+	    d->displayType = dForeign | dTransient | dFromXDMCP;
 	    d->sessionID = pdpy->sessionID;
-	    d->from.data = from_save;
+	    d->from.data = (unsigned char *)from_save;
 	    d->from.length = fromlen;
 	    d->displayNumber = pdpy->displayNumber;
 	    ClientAddress (from, &clientAddress, &clientPort, &connectionType);
@@ -1070,7 +940,7 @@ manage (
 		if (UseChooser (&clientAddress, connectionType))
 		{
 		    d->useChooser = 1;
-		    Debug ("Use chooser for %s\n", d->name);
+		    Debug ("use chooser for %s\n", d->name);
 		}
 	    }
 	    d->clientAddr = clientAddress;
@@ -1091,7 +961,7 @@ manage (
 		pdpy->fileAuthorization = 0;
 	    }
 	    DisposeProtoDisplay (pdpy);
-	    Debug ("Starting display %s,%s\n", d->name, d->class2);
+	    Debug ("starting display %s,%s\n", d->name, d->class2);
 	    StartDisplay (d);
 	    CloseGetter ();
 	}
@@ -1105,9 +975,9 @@ abort:
 void
 SendFailed (
     struct display  *d,
-    char	    *reason)
+    const char	    *reason)
 {
-    Debug ("Display start failed, sending Failed\n");
+    Debug ("display start failed, sending <failed>\n");
     send_failed ((struct sockaddr *)(d->from.data), d->from.length, d->name, d->sessionID, reason);
 }
 
@@ -1115,9 +985,9 @@ static void
 send_failed (
     struct sockaddr *from,
     int		    fromlen,
-    char	    *name,
+    const char	    *name,
     CARD32	    sessionID,
-    char	    *reason)
+    const char	    *reason)
 {
     char	buf[360];
     XdmcpHeader	header;
@@ -1125,7 +995,7 @@ send_failed (
 
     sprintf (buf, "Session %ld failed for display %.260s: %s",
 	     (long) sessionID, name, reason);
-    Debug ("send_failed(\"%s\")", buf);
+    Debug ("send_failed(%\"s)\n", buf);
     status.length = strlen (buf);
     status.data = (CARD8Ptr) buf;
     header.version = XDM_PROTOCOL_VERSION;
@@ -1145,7 +1015,7 @@ send_refuse (
 {
     XdmcpHeader	header;
 
-    Debug ("Send refuse %ld\n", (long) sessionID);
+    Debug ("send <refuse> %ld\n", (long) sessionID);
     header.version = XDM_PROTOCOL_VERSION;
     header.opcode = (CARD16) REFUSE;
     header.length = 4;
@@ -1167,7 +1037,7 @@ send_alive (
     CARD8		sendRunning;
     CARD32		sendSessionID;
 
-    Debug ("Send alive\n");
+    Debug ("send <alive>\n");
     if (XdmcpReadCARD16 (&buffer, &displayNumber) &&
 	XdmcpReadCARD32 (&buffer, &sessionID))
     {
@@ -1180,7 +1050,7 @@ send_alive (
 	    sendRunning = 0;
 	    sendSessionID = 0;
 	    if (d && d->status == running)
- 	    {
+	    {
 		if (d->sessionID == sessionID)
 		    sendRunning = 1;
 		sendSessionID = d->sessionID;
@@ -1188,7 +1058,7 @@ send_alive (
 	    header.version = XDM_PROTOCOL_VERSION;
 	    header.opcode = (CARD16) ALIVE;
 	    header.length = 5;
-	    Debug ("alive: %d %ld\n", sendRunning, (long) sendSessionID);
+	    Debug ("<alive>: %d %ld\n", sendRunning, (long) sendSessionID);
 	    XdmcpWriteHeader (&buffer, &header);
 	    XdmcpWriteCARD8 (&buffer, sendRunning);
 	    XdmcpWriteCARD32 (&buffer, sendSessionID);
@@ -1202,34 +1072,42 @@ NetworkAddressToHostname (
     CARD16	connectionType,
     ARRAY8Ptr   connectionAddress)
 {
-    char    *name = 0;
-
     switch (connectionType)
     {
     case FamilyInternet:
 	{
-	    struct hostent	*hostent;
-	    char dotted[20];
-	    char *local_name;
+	    struct hostent *he;
+	    char *myDot, *name, *lname;
 
-	    hostent = gethostbyaddr ((char *)connectionAddress->data,
-				     connectionAddress->length, AF_INET);
+	    he = gethostbyaddr ((char *)connectionAddress->data, 4, AF_INET);
 
-	    if (hostent)
-		local_name = hostent->h_name;
-	    else {
+	    if (he) {
+		if (StrDup (&name, he->h_name) &&
+		    !strchr (name, '.') &&
+		    (myDot = strchr (localHostname(), '.')))
+		{
+		    if (ASPrintf (&lname, "%s%s", name, myDot))
+		    {
+			if (!(he = gethostbyname (lname)) ||
+			    he->h_addrtype != AF_INET ||
+			    memcmp (he->h_addr, connectionAddress->data, 4))
+			{
+			    free (lname);
+			}
+			else
+			{
+			    free (name);
+			    return lname;
+			}
+		    }
+		}
+	    } else {
 		/* can't get name, so use emergency fallback */
-		sprintf(dotted, "%d.%d.%d.%d",
-			connectionAddress->data[0],
-			connectionAddress->data[1],
-			connectionAddress->data[2],
-			connectionAddress->data[3]);
-		local_name = dotted;
+		ASPrintf (&name, "%[4|'.'hhu", connectionAddress);
 		LogError ("Cannot convert Internet address %s to host name\n",
-			  dotted);
+			  name);
 	    }
-	    StrDup (&name, local_name);
-	    break;
+	    return name;
 	}
 #ifdef DNET
     case FamilyDECnet:
@@ -1238,118 +1116,8 @@ NetworkAddressToHostname (
     default:
 	break;
     }
-    return name;
+    return 0;
 }
 
-#if 0
-static int
-HostnameToNetworkAddress (
-char	    *name,
-CARD16	    connectionType,
-ARRAY8Ptr   connectionAddress)
-{
-    switch (connectionType)
-    {
-    case FamilyInternet:
-	{
-	    struct hostent	*hostent;
-
-	    hostent = gethostbyname (name);
-	    if (!hostent)
-		return FALSE;
-	    if (!XdmcpAllocARRAY8 (connectionAddress, hostent->h_length))
-		return FALSE;
-	    memmove( connectionAddress->data, hostent->h_addr, hostent->h_length);
-	    return TRUE;
-	}
-#ifdef DNET
-    case FamilyDECnet:
-	return FALSE;
-#endif
-    }
-    return FALSE;
-}
-
-/*
- * converts a display name into a network address, using
- * the same rules as XOpenDisplay (algorithm cribbed from there)
- */
-static int
-NameToNetworkAddress(
-char	    *name,
-CARD16Ptr   connectionTypep,
-ARRAY8Ptr   connectionAddress,
-CARD16Ptr   displayNumber)
-{
-    char    *colon, *display_number;
-    char    hostname[1024];
-    int	    dnet = FALSE;
-    CARD16  number;
-    CARD16  connectionType;
-
-    colon = strchr(name, ':');
-    if (!colon)
-	return FALSE;
-    if (colon != name)
-    {
-	if (colon - name > sizeof (hostname))
-	    return FALSE;
-	strncpy (hostname, name, colon - name);
-	hostname[colon - name] = '\0';
-    }
-    else
-    {
-	strcpy (hostname, localHostname ());
-    }
-    if (colon[1] == ':')
-    {
-	dnet = TRUE;
-	colon++;
-    }
-#ifndef DNETCONN
-    if (dnet)
-	return FALSE;
-#endif
-    display_number = colon + 1;
-    while (*display_number && *display_number != '.')
-    {
-	if (!isascii (*display_number) || !isdigit(*display_number))
-	    return FALSE;
-    }
-    if (display_number == colon + 1)
-	return FALSE;
-    number = atoi (colon + 1);
-#ifdef DNETCONN
-    if (dnet)
-	connectionType = FamilyDECnet;
-    else
-#endif
-	connectionType = FamilyInternet;
-    if (!HostnameToNetworkAddress (hostname, connectionType, connectionAddress))
-	return FALSE;
-    *displayNumber = number;
-    *connectionTypep = connectionType;
-    return TRUE;
-}
-#endif
-
-#ifdef MINIX
-void udp_read_cb(nbio_ref_t ref, int res, int err)
-{
-	if (!read_inprogress)
-		abort();
-	if (res > 0)
-	{
-		read_size= res;
-	}
-	else
-    	{
-    		LogError("read error: %s\n",
-    			res == 0 ?  "EOF" : strerror(err));
-		read_size= 0;
-	}
-	read_inprogress= 0;
-}
-#endif
-
 #endif /* XDMCP */
+
