@@ -5,6 +5,7 @@
 
   Copyright (c) 2000 Matthias Hoelzer-Kluepfel <hoelzer@kde.org>
                      Stefan Schimanski <1Stein@gmx.de>
+                2003 George Staikos <staikos@kde.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -18,7 +19,7 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 */
 
@@ -29,22 +30,31 @@
 #include <stdlib.h>
 #include <unistd.h>
 
-#include <qdir.h>
 #include <qdict.h>
+#include <qdir.h>
+#include <qfile.h>
 #include <qtimer.h>
 
 #include "kxt.h"
 #include "nsplugin.h"
 #include "resolve.h"
 
-#include <klibloader.h>
-#include <kdebug.h>
-#include <kurl.h>
-#include <kio/netaccess.h>
-#include <ktempfile.h>
+#ifdef Bool
+#undef Bool
+#endif
+
 #include <dcopclient.h>
-#include <kprotocolmanager.h>
+#include <kconfig.h>
+#include <kdebug.h>
+#include <kglobal.h>
+#include <kio/netaccess.h>
+#include <klibloader.h>
 #include <klocale.h>
+#include <kprocess.h>
+#include <kprotocolmanager.h>
+#include <kstddirs.h>
+#include <ktempfile.h>
+#include <kurl.h>
 
 #include <X11/Intrinsic.h>
 #include <X11/Composite.h>
@@ -53,6 +63,43 @@
 #include <X11/StringDefs.h>
 #include <Xm/DrawingA.h>
 
+// provide these symbols when compiling with gcc 3.x
+
+#if defined __GNUC__ && defined __GNUC_MINOR__
+# define KDE_GNUC_PREREQ(maj, min) \
+  ((__GNUC__ << 16) + __GNUC_MINOR__ >= ((maj) << 16) + (min))
+#else
+# define KDE_GNUC_PREREQ(maj, min) 0
+#endif
+
+
+#if defined(__GNUC__) && KDE_GNUC_PREREQ(3,0)
+extern "C" void* __builtin_new(size_t s)
+{
+   return operator new(s);
+}
+
+extern "C" void __builtin_delete(void* p)
+{
+   operator delete(p);
+}
+
+extern "C" void* __builtin_vec_new(size_t s)
+{
+   return operator new[](s);
+}
+
+extern "C" void __builtin_vec_delete(void* p)
+{
+   operator delete[](p);
+}
+
+extern "C" void __pure_virtual()
+{
+   abort();
+}
+#endif
+
 // server side functions -----------------------------------------------------
 
 // allocate memory
@@ -60,7 +107,7 @@ void *g_NPN_MemAlloc(uint32 size)
 {
    void *mem = ::malloc(size);
 
-   kdDebug(1431) << "g_NPN_MemAlloc(), size=" << size << " allocated at " << mem << endl;
+   //kdDebug(1431) << "g_NPN_MemAlloc(), size=" << size << " allocated at " << mem << endl;
 
    return mem;
 }
@@ -69,14 +116,16 @@ void *g_NPN_MemAlloc(uint32 size)
 // free memory
 void g_NPN_MemFree(void *ptr)
 {
-   kdDebug(1431) << "g_NPN_MemFree() at " << ptr << endl;
+   //kdDebug(1431) << "g_NPN_MemFree() at " << ptr << endl;
    if (ptr)
      ::free(ptr);
 }
 
-uint32 g_NPN_MemFlush(uint32 /*size*/)
+uint32 g_NPN_MemFlush(uint32 size)
 {
-   kdDebug(1431) << "g_NPN_MemFlush()" << endl;
+   Q_UNUSED(size);
+   //kdDebug(1431) << "g_NPN_MemFlush()" << endl;
+   // MAC OS only..  we don't use this
    return 0;
 }
 
@@ -84,13 +133,17 @@ uint32 g_NPN_MemFlush(uint32 /*size*/)
 // redraw
 void g_NPN_ForceRedraw(NPP /*instance*/)
 {
+   // http://devedge.netscape.com/library/manuals/2002/plugin/1.0/npn_api3.html#999401
+   // FIXME
    kdDebug(1431) << "g_NPN_ForceRedraw() [unimplemented]" << endl;
 }
 
 
 // invalidate rect
-void g_NPN_InvalidateRect(NPP /*instance*/, NPRect */*invalidRect*/)
+void g_NPN_InvalidateRect(NPP /*instance*/, NPRect* /*invalidRect*/)
 {
+   // http://devedge.netscape.com/library/manuals/2002/plugin/1.0/npn_api7.html#999503
+   // FIXME
    kdDebug(1431) << "g_NPN_InvalidateRect() [unimplemented]" << endl;
 }
 
@@ -98,6 +151,8 @@ void g_NPN_InvalidateRect(NPP /*instance*/, NPRect */*invalidRect*/)
 // invalidate region
 void g_NPN_InvalidateRegion(NPP /*instance*/, NPRegion /*invalidRegion*/)
 {
+   // http://devedge.netscape.com/library/manuals/2002/plugin/1.0/npn_api8.html#999528
+   // FIXME
    kdDebug(1431) << "g_NPN_InvalidateRegion() [unimplemented]" << endl;
 }
 
@@ -115,17 +170,17 @@ NPError g_NPN_GetValue(NPP /*instance*/, NPNVariable variable, void *value)
       case NPNVxtAppContext:
          *(void**)value = XtDisplayToApplicationContext(qt_xdisplay());
          return NPERR_NO_ERROR;
-#ifdef NP4
       case NPNVjavascriptEnabledBool:
-         (bool)(*value) = false; // FIXME: Let's have a talk with Harri :-)
+         *(bool*)value = true;
          return NPERR_NO_ERROR;
       case NPNVasdEnabledBool:
-         (bool)(*value) = false; // FIXME: No clue what this means...
+         // SmartUpdate - we don't do this
+         *(bool*)value = false;
          return NPERR_NO_ERROR;
-      case NPNVOfflineBool:
-         (bool)(*value) = false;
+      case NPNVisOfflineBool:
+         // Offline browsing - no thanks
+         *(bool*)value = false;
          return NPERR_NO_ERROR;
-#endif
       default:
          return NPERR_INVALID_PARAM;
    }
@@ -133,36 +188,54 @@ NPError g_NPN_GetValue(NPP /*instance*/, NPNVariable variable, void *value)
 
 
 NPError g_NPN_DestroyStream(NPP instance, NPStream* stream,
-                          NPReason /*reason*/)
+                          NPReason reason)
 {
+   // FIXME: is this correct?  I imagine it is not.  (GS)
    kdDebug(1431) << "g_NPN_DestroyStream()" << endl;
 
    NSPluginInstance *inst = (NSPluginInstance*) instance->ndata;
    inst->streamFinished( (NSPluginStream *)stream->ndata );
 
-   return NPERR_GENERIC_ERROR;
+   switch (reason) {
+   case NPRES_DONE:
+      return NPERR_NO_ERROR;
+   case NPRES_USER_BREAK:
+      // FIXME: notify the user
+   case NPRES_NETWORK_ERR:
+      // FIXME: notify the user
+   default:
+      return NPERR_GENERIC_ERROR;
+   }
 }
 
 
-NPError g_NPN_RequestRead(NPStream */*stream*/, NPByteRange */*rangeList*/)
+NPError g_NPN_RequestRead(NPStream* /*stream*/, NPByteRange* /*rangeList*/)
 {
+   // http://devedge.netscape.com/library/manuals/2002/plugin/1.0/npn_api16.html#999734
    kdDebug(1431) << "g_NPN_RequestRead() [unimplemented]" << endl;
 
+   // FIXME
    return NPERR_GENERIC_ERROR;
 }
 
 NPError g_NPN_NewStream(NPP /*instance*/, NPMIMEType /*type*/,
                       const char* /*target*/, NPStream** /*stream*/)
 {
+   // http://devedge.netscape.com/library/manuals/2002/plugin/1.0/npn_api12.html#999628
    kdDebug(1431) << "g_NPN_NewStream() [unimplemented]" << endl;
 
+   // FIXME
+   // This creates a stream from the plugin to the browser of type "type" to
+   // display in "target"
    return NPERR_GENERIC_ERROR;
 }
 
-int32 g_NPN_Write(NPP /*instance*/, NPStream */*stream*/, int32 /*len*/, void */*buf*/)
+int32 g_NPN_Write(NPP /*instance*/, NPStream* /*stream*/, int32 /*len*/, void* /*buf*/)
 {
+   // http://devedge.netscape.com/library/manuals/2002/plugin/1.0/npn_api21.html#999859
    kdDebug(1431) << "g_NPN_Write() [unimplemented]" << endl;
 
+   // FIXME
    return 0;
 }
 
@@ -172,7 +245,7 @@ NPError g_NPN_GetURL(NPP instance, const char *url, const char *target)
 {
    kdDebug(1431) << "g_NPN_GetURL: url=" << url << " target=" << target << endl;
 
-   NSPluginInstance *inst = (NSPluginInstance*) instance->ndata;
+   NSPluginInstance *inst = static_cast<NSPluginInstance*>(instance->ndata);
    inst->requestURL( QString::fromLatin1(url), QString::null,
                      QString::fromLatin1(target), 0 );
 
@@ -184,7 +257,7 @@ NPError g_NPN_GetURLNotify(NPP instance, const char *url, const char *target,
                          void* notifyData)
 {
     kdDebug(1431) << "g_NPN_GetURLNotify: url=" << url << " target=" << target << " inst=" << (void*)instance << endl;
-   NSPluginInstance *inst = (NSPluginInstance*) instance->ndata;
+   NSPluginInstance *inst = static_cast<NSPluginInstance*>(instance->ndata);
    kdDebug(1431) << "g_NPN_GetURLNotify: ndata=" << (void*)inst << endl;
    inst->requestURL( QString::fromLatin1(url), QString::null,
                      QString::fromLatin1(target), notifyData );
@@ -193,21 +266,110 @@ NPError g_NPN_GetURLNotify(NPP instance, const char *url, const char *target,
 }
 
 
-NPError g_NPN_PostURL(NPP /*instance*/, const char */*url*/, const char */*target*/,
-                    uint32 /*len*/, const char */*buf*/, NPBool /*file*/)
+NPError g_NPN_PostURLNotify(NPP instance, const char* url, const char* target,
+                     uint32 len, const char* buf, NPBool file, void* notifyData)
 {
-   kdDebug(1431) << "g_NPN_PostURL() [unimplemented]" << endl;
+// http://devedge.netscape.com/library/manuals/2002/plugin/1.0/npn_api14.html
+   kdDebug(1431) << "g_NPN_PostURLNotify() [incomplete]" << endl;
+   kdDebug(1431) << "url=[" << url << "] target=[" << target << "]" << endl;
+   QByteArray postdata;
+   KParts::URLArgs args;
 
-   return NPERR_GENERIC_ERROR;
+   if (len == 0) {
+      return NPERR_NO_DATA;
+   }
+
+   if (file) { // buf is a filename
+      QFile f(buf);
+      if (!f.open(IO_ReadOnly)) {
+         return NPERR_FILE_NOT_FOUND;
+      }
+
+      // FIXME: this will not work because we need to strip the header out!
+      postdata = f.readAll();
+      f.close();
+   } else {    // buf is raw data
+      // First strip out the header
+      const char *previousStart = buf;
+      uint32 l;
+      bool previousCR = true;
+
+      for (l = 1;; l++) {
+         if (l == len) {
+            break;
+         }
+
+         if (buf[l-1] == '\n' || (previousCR && buf[l-1] == '\r')) {
+            if (previousCR) { // header is done!
+               if ((buf[l-1] == '\r' && buf[l] == '\n') ||
+                   (buf[l-1] == '\n' &&  buf[l] == '\r'))
+                  l++;
+               l++;
+               previousStart = &buf[l-1];
+               break;
+            }
+
+            QString thisLine = QString::fromLatin1(previousStart, &buf[l-1] - previousStart).stripWhiteSpace();
+
+            previousStart = &buf[l];
+            previousCR = true;
+
+            kdDebug(1431) << "Found header line: [" << thisLine << "]" << endl;
+            if (thisLine.startsWith("Content-Type: ")) {
+               args.setContentType(thisLine);
+            }
+         } else {
+            previousCR = false;
+         }
+      }
+
+      postdata.duplicate(previousStart, len - l + 1);
+   }
+
+   kdDebug(1431) << "Post data: " << postdata.size() << " bytes" << endl;
+#if 0
+   QFile f("/tmp/nspostdata");
+   f.open(IO_WriteOnly);
+   f.writeBlock(postdata);
+   f.close();
+#endif
+
+   if (!target || !*target) {
+      // Send the results of the post to the plugin
+      // (works by default)
+   } else if (!strcmp(target, "_current") || !strcmp(target, "_self") ||
+              !strcmp(target, "_top")) {
+      // Unload the plugin, put the results in the frame/window that the
+      // plugin was loaded in
+      // FIXME
+   } else if (!strcmp(target, "_new") || !strcmp(target, "_blank")){
+      // Open a new browser window and write the results there
+      // FIXME
+   } else {
+      // Write the results to the specified frame
+      // FIXME
+   }
+
+   NSPluginInstance *inst = static_cast<NSPluginInstance*>(instance->ndata);
+   if (inst && !inst->normalizedURL(QString::fromLatin1(url)).isNull()) {
+      inst->postURL( QString::fromLatin1(url), postdata, args.contentType(),
+                     QString::fromLatin1(target), notifyData, args );
+   } else {
+      // Unsupported / insecure
+      return NPERR_INVALID_URL;
+   }
+
+   return NPERR_NO_ERROR;
 }
 
 
-NPError g_NPN_PostURLNotify(NPP /*instance*/, const char */*url*/, const char */*target*/,
-                          uint32 /*len*/, const char */*buf*/, NPBool /*file*/, void */*notifyData*/)
+NPError g_NPN_PostURL(NPP instance, const char* url, const char* target,
+                    uint32 len, const char* buf, NPBool file)
 {
-   kdDebug(1431) << "g_NPN_PostURL() [unimplemented]" << endl;
+// http://devedge.netscape.com/library/manuals/2002/plugin/1.0/npn_api13.html
+   kdDebug(1431) << "g_NPN_PostURL()" << endl;
 
-   return NPERR_GENERIC_ERROR;
+   return g_NPN_PostURLNotify(instance, url, target, len, buf, file, 0L);
 }
 
 
@@ -250,9 +412,25 @@ void g_NPN_Version(int *plugin_major, int *plugin_minor, int *browser_major, int
 }
 
 
-void g_NPN_ReloadPlugins(NPBool /*reloadPages*/)
+void g_NPN_ReloadPlugins(NPBool reloadPages)
 {
-   kdDebug(1431) << "g_NPN_ReloadPlugins() [unimplemented]" << endl;
+   // http://devedge.netscape.com/library/manuals/2002/plugin/1.0/npn_api15.html#999713
+   kdDebug(1431) << "g_NPN_ReloadPlugins()" << endl;
+   KProcess p;
+   p << KGlobal::dirs()->findExe("nspluginscan");
+
+   if (reloadPages) {
+      // This is the proper way, but it cannot be done because we have no
+      // handle to the caller!  How stupid!  We cannot force all konqi windows
+      // to reload - that would be evil.
+      //p.start(KProcess::Block);
+      // Let's only allow the caller to be reloaded, not everything.
+      //if (_callback)
+      //   _callback->reloadPage();
+      p.start(KProcess::DontCare);
+   } else {
+      p.start(KProcess::DontCare);
+   }
 }
 
 
@@ -260,6 +438,8 @@ void g_NPN_ReloadPlugins(NPBool /*reloadPages*/)
 JRIEnv *g_NPN_GetJavaEnv()
 {
    kdDebug(1431) << "g_NPN_GetJavaEnv() [unimplemented]" << endl;
+   // FIXME - what do these do?  I can't find docs, and even Mozilla doesn't
+   //         implement them
    return 0;
 }
 
@@ -267,15 +447,25 @@ JRIEnv *g_NPN_GetJavaEnv()
 jref g_NPN_GetJavaPeer(NPP /*instance*/)
 {
    kdDebug(1431) << "g_NPN_GetJavaPeer() [unimplemented]" << endl;
+   // FIXME - what do these do?  I can't find docs, and even Mozilla doesn't
+   //         implement them
    return 0;
 }
 
 
-NPError g_NPN_SetValue(NPP /*instance*/, NPPVariable /*variable*/, void */*value*/)
+NPError g_NPN_SetValue(NPP /*instance*/, NPPVariable variable, void* /*value*/)
 {
    kdDebug(1431) << "g_NPN_SetValue() [unimplemented]" << endl;
-
-   return NPERR_GENERIC_ERROR;
+   switch (variable) {
+   case NPPVpluginWindowBool:
+      // FIXME
+      // If true, the plugin is windowless.  If false, it is in a window.
+   case NPPVpluginTransparentBool:
+      // FIXME
+      // If true, the plugin is displayed transparent
+   default:
+      return NPERR_GENERIC_ERROR;
+   }
 }
 
 
@@ -284,13 +474,31 @@ NPError g_NPN_SetValue(NPP /*instance*/, NPPVariable /*variable*/, void */*value
 
 /******************************************************************/
 
+void
+NSPluginInstance::forwarder(Widget w, XtPointer cl_data, XEvent * event, Boolean * cont)
+{
+  Q_UNUSED(w);
+  NSPluginInstance *inst = (NSPluginInstance*)cl_data;
+  *cont = True;
+  if (inst->_area == 0 || event->xkey.window == XtWindow(inst->_area))
+    return;
+  *cont = False;
+  event->xkey.window = XtWindow(inst->_area);
+  event->xkey.subwindow = None;
+  XtDispatchEvent(event);
+}
+
+
 NSPluginInstance::NSPluginInstance(NPP privateData, NPPluginFuncs *pluginFuncs,
                                    KLibrary *handle, int width, int height,
                                    QString src, QString /*mime*/,
                                    QString appId, QString callbackId,
+                                   bool embed,
                                    QObject *parent, const char* name )
-   : QObject( parent, name ), DCOPObject()
+   : DCOPObject(), QObject( parent, name ) 
 {
+    Q_UNUSED(embed);
+   _visible = false;
    _npp = privateData;
    _npp->ndata = this;
    _destroyed = false;
@@ -315,6 +523,12 @@ NSPluginInstance::NSPluginInstance(NPP privateData, NPPluginFuncs *pluginFuncs,
    kdDebug(1431) << "pdata = " << _npp->pdata << endl;
    kdDebug(1431) << "ndata = " << _npp->ndata << endl;
 
+   if (width == 0)
+      width = 1600;
+
+   if (height == 0)
+      height = 1200;
+
    // create drawing area
    Arg args[5];
    Cardinal nargs=0;
@@ -329,7 +543,8 @@ NSPluginInstance::NSPluginInstance(NPP privateData, NPPluginFuncs *pluginFuncs,
 
    // What exactly does widget mapping mean? Without this call the widget isn't
    // embedded correctly. With it the viewer doesn't show anything in standalone mode.
-   XtSetMappedWhenManaged(_toplevel, False);
+   //if (embed)
+      XtSetMappedWhenManaged(_toplevel, False);
    XtRealizeWidget(_toplevel);
 
    // Create form window that is searched for by flash plugin
@@ -337,9 +552,16 @@ NSPluginInstance::NSPluginInstance(NPP privateData, NPPluginFuncs *pluginFuncs,
    XtRealizeWidget(_form);
 
    // Create widget that is passed to the plugin
+   XtInitializeWidgetClass(xmDrawingAreaWidgetClass);
    _area = XmCreateDrawingArea( _form, (char*)("drawingArea"), args, nargs);
    XtRealizeWidget(_area);
    XtMapWidget(_area);
+   
+   // Register forwarder
+   XtAddEventHandler(_toplevel, (KeyPressMask|KeyReleaseMask), 
+                     False, forwarder, (XtPointer)this );
+   XtAddEventHandler(_form, (KeyPressMask|KeyReleaseMask), 
+                     False, forwarder, (XtPointer)this );
 }
 
 NSPluginInstance::~NSPluginInstance()
@@ -357,6 +579,8 @@ void NSPluginInstance::destroy()
         kdDebug(1431) << "delete streams" << endl;
         _waitingRequests.clear();
 
+	shutdown();
+
         for( NSPluginStreamBase *s=_streams.first(); s!=0; ) {
             NSPluginStreamBase *next = _streams.next();
             s->stop();
@@ -371,24 +595,35 @@ void NSPluginInstance::destroy()
 
         kdDebug(1431) << "destroy plugin" << endl;
         NPSavedData *saved = 0;
-#if 0
-// Disabled for 2.2 release since it causes nsplugin to crash when
-// used with Qt linked with libGL.  This will create some sort of
-// memory leak.. but better a leak in some cases than a constantly
-// crashing nspluginviewer
+
+        // As of 7/31/01, nsplugin crashes when used with Qt
+        // linked with libGL if the destroy function is called.
+        // A patch on that date hacked out the following call.
+        // On 11/17/01, Jeremy White has reenabled this destroy
+        // in a an attempt to better understand why this crash
+        // occurs so that the real problem can be found and solved.
+        // It's possible that a flaw in the SetWindow call
+        // caused the crash and it is now fixed.
         if ( _pluginFuncs.destroy )
             _pluginFuncs.destroy( _npp, &saved );
-#endif
+
         if (saved && saved->len && saved->buf)
           g_NPN_MemFree(saved->buf);
         if (saved)
           g_NPN_MemFree(saved);
 
+        XtRemoveEventHandler(_form, (KeyPressMask|KeyReleaseMask), 
+                             False, forwarder, (XtPointer)this);
+        XtRemoveEventHandler(_toplevel, (KeyPressMask|KeyReleaseMask), 
+                             False, forwarder, (XtPointer)this);
         XtDestroyWidget(_area);
+	_area = 0;
         XtDestroyWidget(_form);
+	_form = 0;
         XtDestroyWidget(_toplevel);
+	_toplevel = 0;
 
-        ::free(_npp);
+        ::free(_npp);   // matched with malloc() in newInstance
 
         _destroyed = true;
     }
@@ -398,13 +633,18 @@ void NSPluginInstance::destroy()
 void NSPluginInstance::shutdown()
 {
     NSPluginClass *cls =  static_cast<NSPluginClass*>(parent());
-    destroy();
+    //destroy();
     cls->destroyInstance( this );
 }
 
 
 void NSPluginInstance::timer()
 {
+    if (!_visible) {
+         _timer->start( 100, true );
+         return;
+    }
+
     _streams.clear();
 
     // start queued requests
@@ -430,22 +670,38 @@ void NSPluginInstance::timer()
             url = req.url;
 
         // non empty target = frame target
-        if ( !req.target.isEmpty() )
+        if ( !req.target.isEmpty())
         {
             if (_callback)
             {
-                _callback->requestURL( url, req.target );
-                if ( req.notify )
+                if ( req.post ) {
+                    _callback->postURL( url, req.target, req.data, req.mime );
+                } else {
+                    _callback->requestURL( url, req.target );
+                }
+                if ( req.notify ) {
                     NPURLNotify( req.url, NPRES_DONE, req.notify );
+                }
             }
         } else {
             if (!url.isEmpty())
             {
                 kdDebug(1431) << "Starting new stream " << req.url << endl;
 
-                // hack to get java vm and HyperCosm 3d working
-                if ( url.lower()=="javascript:document.location" ||
-                     url.lower()=="javascript:window.location.href" ) {
+                if (req.post) {
+                    // create stream
+                    NSPluginStream *s = new NSPluginStream( this );
+                    connect( s, SIGNAL(finished(NSPluginStreamBase*)),
+                             SLOT(streamFinished(NSPluginStreamBase*)) );
+                    _streams.append( s );
+
+                    kdDebug() << "posting to " << url << endl;
+
+                    emitStatus( i18n("Submitting data to %1").arg(url) );
+                    s->post( url, req.data, req.mime, req.notify, req.args );
+                } else if (url.lower().startsWith("javascript:document.location") ||
+                     url.lower().startsWith("javascript:window.location.href")) {
+                    // hack to get java vm and HyperCosm 3d working
                     NSPluginBufStream *s = new NSPluginBufStream( this );
                     connect( s, SIGNAL(finished(NSPluginStreamBase*)),
                              SLOT(streamFinished(NSPluginStreamBase*)) );
@@ -454,6 +710,13 @@ void NSPluginInstance::timer()
                     QByteArray buf;
                     buf.setRawData( _baseURL.latin1(), _baseURL.length()+1 );
                     s->get( url, "text/html", buf, req.notify, true );
+                } else if (url.lower().startsWith("javascript:")){
+                    if (_callback) {
+                        static int _jsrequestid = 0;
+                        _callback->evalJavaScript( _jsrequestid, url.mid(11) );
+                        if ( req.notify )
+                            _jsrequests.insert(_jsrequestid++, new Request( req ));
+                    }
                 } else {
                     // create stream
                     NSPluginStream *s = new NSPluginStream( this );
@@ -474,14 +737,57 @@ void NSPluginInstance::timer()
 }
 
 
+QString NSPluginInstance::normalizedURL(const QString& url) const {
+    KURL inURL(_baseURL, url);
+    KConfig cfg("kcmnspluginrc", true);
+    cfg.setGroup("Misc");
+
+    if (!cfg.readBoolEntry("HTTP URLs Only", false) ||
+	inURL.protocol() == "http" ||
+        inURL.protocol() == "https" ||
+        inURL.protocol() == "javascript") {
+        return inURL.url();
+    }
+
+    // Allow: javascript:, http, https, or no protocol (match loading)
+    kdDebug(1431) << "NSPluginInstance::normalizedURL - I don't think so.  http or https only!" << endl;
+    return QString::null;
+}
+
+
 void NSPluginInstance::requestURL( const QString &url, const QString &mime,
                                    const QString &target, void *notify )
 {
-    kdDebug(1431) << "NSPluginInstance::requestURL url=" << url << " target=" <<
-        target << " notify=" << notify << endl;
-    _waitingRequests.enqueue( new Request( url, mime, target, notify ) );
-    if ( _streams.count()==0 )
+    // Generally this should already be done, but let's be safe for now.
+    QString nurl = normalizedURL(url);
+    if (nurl.isNull()) {
+        return;
+    }
+
+    kdDebug(1431) << "NSPluginInstance::requestURL url=" << nurl << " target=" << target << " notify=" << notify << endl;
+    _waitingRequests.enqueue( new Request( nurl, mime, target, notify ) );
+    if ( _streams.count() == 0 ) {
         _timer->start( 0, true );
+    }
+}
+
+
+void NSPluginInstance::postURL( const QString &url, const QByteArray& data,
+                                const QString &mime,
+                                const QString &target, void *notify,
+                                const KParts::URLArgs& args )
+{
+    // Generally this should already be done, but let's be safe for now.
+    QString nurl = normalizedURL(url);
+    if (nurl.isNull()) {
+        return;
+    }
+
+    kdDebug(1431) << "NSPluginInstance::postURL url=" << nurl << " target=" << target << " notify=" << notify << endl;
+    _waitingRequests.enqueue( new Request( nurl, data, mime, target, notify, args) );
+    if ( _streams.count() == 0 ) {
+        _timer->start( 0, true );
+    }
 }
 
 
@@ -492,7 +798,7 @@ void NSPluginInstance::emitStatus(const QString &message)
 }
 
 
-void NSPluginInstance::streamFinished( NSPluginStreamBase */*strm*/ )
+void NSPluginInstance::streamFinished( NSPluginStreamBase* /*strm*/ )
 {
    kdDebug(1431) << "-> NSPluginInstance::streamFinished" << endl;
    emitStatus( QString::null );
@@ -510,44 +816,60 @@ int NSPluginInstance::setWindow(int remove)
 
    kdDebug(1431) << "-> NSPluginInstance::setWindow" << endl;
 
-   NPWindow win;
-   NPSetWindowCallbackStruct win_info;
-
-   win.x = 0;
-   win.y = 0;
-   win.height = _height;
-   win.width = _width;
-   win.type = NPWindowTypeWindow;
+   _win.x = 0;
+   _win.y = 0;
+   _win.height = _height;
+   _win.width = _width;
+   _win.type = NPWindowTypeWindow;
 
    // Well, the docu says sometimes, this is only used on the
    // MAC, but sometimes it says it's always. Who knows...
-   NPRect clip;
-   clip.top = 0;
-   clip.left = 0;
-   clip.bottom = _height;
-   clip.right = _width;
-   win.clipRect = clip;
+   _win.clipRect.top = 0;
+   _win.clipRect.left = 0;
+   _win.clipRect.bottom = _height;
+   _win.clipRect.right = _width;
 
-   win.window = (void*) XtWindow(_area);
-   kdDebug(1431) << "Window ID = " << win.window << endl;
+   _win.window = (void*) XtWindow(_area);
+   kdDebug(1431) << "Window ID = " << _win.window << endl;
 
-   win_info.type = NP_SETWINDOW;
-   win_info.display = XtDisplay( _area );
-   win_info.visual = DefaultVisualOfScreen( XtScreen(_area) );
-   win_info.colormap = DefaultColormapOfScreen( XtScreen(_area) );
-   win_info.depth = DefaultDepthOfScreen( XtScreen(_area) );
+   _win_info.type = NP_SETWINDOW;
+   _win_info.display = XtDisplay( _area );
+   _win_info.visual = DefaultVisualOfScreen( XtScreen(_area) );
+   _win_info.colormap = DefaultColormapOfScreen( XtScreen(_area) );
+   _win_info.depth = DefaultDepthOfScreen( XtScreen(_area) );
 
-   win.ws_info = &win_info;
+   _win.ws_info = &_win_info;
 
-   NPError error = NPSetWindow( &win );
+   NPError error = NPSetWindow( &_win );
 
    kdDebug(1431) << "<- NSPluginInstance::setWindow = " << error << endl;
    return error;
 }
 
 
+static void resizeWidgetTree(Window w, int width, int height) {
+   Window rroot, parent, *children;
+   unsigned int nchildren = 0;
+
+   if (XQueryTree(qt_xdisplay(), w, &rroot, &parent, &children, &nchildren)) {
+      for (unsigned int i = 0; i < nchildren; i++) {
+         XResizeWindow(qt_xdisplay(), children[i], width, height);
+         resizeWidgetTree(children[i], width, height);
+      }
+      XFree(children);
+   }
+
+}
+
+
 void NSPluginInstance::resizePlugin(int w, int h)
 {
+   if (!_visible)
+      return;
+
+   if (w == _width && h == _height)
+      return;
+
    kdDebug(1431) << "-> NSPluginInstance::resizePlugin( w=" << w << ", h=" << h << " ) " << endl;
 
    _width = w;
@@ -561,12 +883,37 @@ void NSPluginInstance::resizePlugin(int w, int h)
    XtSetValues(_form, args, nargs);
    XtSetValues(_area, args, nargs);
 
+   resizeWidgetTree(XtWindow(_area), _width, _height);
+
    setWindow();
 
    kdDebug(1431) << "<- NSPluginInstance::resizePlugin" << endl;
 }
 
 
+void NSPluginInstance::javascriptResult(int id, QString result) {
+    QMap<int, Request*>::iterator i = _jsrequests.find( id );
+    if (i != _jsrequests.end()) {
+        Request *req = i.data();
+        _jsrequests.remove( i );
+        NSPluginStream *s = new NSPluginStream( this );
+        connect( s, SIGNAL(finished(NSPluginStreamBase*)),
+                 SLOT(streamFinished(NSPluginStreamBase*)) );
+        _streams.append( s );
+
+        s->create( req->url, QString("text/plain"), req->notify );
+        int len = result.length();
+        QByteArray data( len + 1 );
+        if (len)
+            memcpy( data.data(), result.latin1(), len );
+        data[len] = 0;
+        s->process( data, 0 );
+        s->finish( false );
+        delete req;
+    }
+}
+
+
 NPError NSPluginInstance::NPGetValue(NPPVariable variable, void *value)
 {
     if( value==0 ) {
@@ -718,11 +1065,26 @@ void NSPluginInstance::addTempFile(KTempFile *tmpFile)
    _tempFiles.append(tmpFile);
 }
 
+/*
+ *   We have to call this after we reparent the widget otherwise some plugins
+ *   like the ones based on WINE get very confused. (their coordinates are not
+ *   adjusted for the mouse at best)
+ */
+void NSPluginInstance::displayPlugin()
+{
+   // display plugin
+   setWindow();
+
+   _visible = true;
+   kdDebug(1431) << "<- NSPluginInstance::displayPlugin = " << (void*)this << endl;
+}
+
+
 /***************************************************************************/
 
 NSPluginViewer::NSPluginViewer( QCString dcopId,
                                 QObject *parent, const char *name )
-   : QObject( parent, name ), DCOPObject(dcopId)
+   : DCOPObject(dcopId), QObject( parent, name ) 
 {
     _classes.setAutoDelete( true );
 }
@@ -738,7 +1100,11 @@ void NSPluginViewer::shutdown()
 {
    kdDebug(1431) << "NSPluginViewer::shutdown" << endl;
    _classes.clear();
+#if QT_VERSION < 0x030100
    quitXt();
+#else
+   qApp->quit();
+#endif
 }
 
 
@@ -769,10 +1135,10 @@ DCOPRef NSPluginViewer::newClass( QString plugin )
 
 NSPluginClass::NSPluginClass( const QString &library,
                               QObject *parent, const char *name )
-   : QObject( parent, name ), DCOPObject()
+   : DCOPObject(), QObject( parent, name ) 
 {
     // initialize members
-    _handle = KLibLoader::self()->library(library.latin1());
+    _handle = KLibLoader::self()->library(QFile::encodeName(library));
     _libname = library;
     _constructed = false;
     _error = true;
@@ -823,8 +1189,8 @@ NSPluginClass::~NSPluginClass()
     _instances.clear();
     _trash.clear();
     shutdown();
-    //KLibLoader::self()->unloadLibrary( _libname.latin1() );
-    delete _handle;
+    if (_handle)
+      _handle->unload();
 }
 
 
@@ -888,6 +1254,7 @@ QString NSPluginClass::getMIMEDescription()
 
 void NSPluginClass::shutdown()
 {
+    kdDebug(1431) << "NSPluginClass::shutdown error=" << _error << endl;
     if( _NP_Shutdown && !_error )
         _NP_Shutdown();
 }
@@ -907,8 +1274,8 @@ DCOPRef NSPluginClass::newInstance( QString url, QString mimeType, bool embed,
    char **_argn = new char*[argc];
    char **_argv = new char*[argc];
    QString src = url;
-   int width = 300;
-   int height = 300;
+   int width = 0;
+   int height = 0;
    QString baseURL = url;
 
    for (unsigned int i=0; i<argc; i++)
@@ -919,22 +1286,27 @@ DCOPRef NSPluginClass::newInstance( QString url, QString mimeType, bool embed,
       _argn[i] = strdup(n);
       _argv[i] = strdup(v);
 
-      if (!strcasecmp(_argn[i], "WIDTH")) width = atoi(_argv[i]);
-      if (!strcasecmp(_argn[i], "HEIGHT")) height = atoi(_argv[i]);
+      if (!strcasecmp(_argn[i], "WIDTH")) width = argv[i].toInt();
+      if (!strcasecmp(_argn[i], "HEIGHT")) height = argv[i].toInt();
       if (!strcasecmp(_argn[i], "__KHTML__PLUGINBASEURL")) baseURL = _argv[i];
       kdDebug(1431) << "argn=" << _argn[i] << " argv=" << _argv[i] << endl;
    }
 
    // create plugin instance
    char mime[256];
-   strcpy(mime, mimeType.ascii());
-   NPP npp = new NPP_t;
+   strncpy(mime, mimeType.ascii(), 255);
+   mime[255] = 0;
+   NPP npp = (NPP)malloc(sizeof(NPP_t));   // I think we should be using
+                                           // malloc here, just to be safe,
+                                           // since the nsplugin plays with
+                                           // this thing
+   memset(npp, 0, sizeof(NPP_t));
    npp->ndata = NULL;
 
    // Create plugin instance object
    NSPluginInstance *inst = new NSPluginInstance( npp, &_pluginFuncs, _handle,
                                                   width, height, baseURL, mimeType,
-                                                  appId, callbackId, this );
+                                                  appId, callbackId, embed, this );
 
    // create plugin instance
    NPError error = _pluginFuncs.newp(mime, npp, embed ? NP_EMBED : NP_FULL,
@@ -949,18 +1321,15 @@ DCOPRef NSPluginClass::newInstance( QString url, QString mimeType, bool embed,
    if ( error!=NPERR_NO_ERROR)
    {
       delete inst;
-      delete npp;
-      kdDebug(1431) << "<- PlluginClass::NewInstance = 0" << endl;
+      //delete npp;    double delete!
+      kdDebug(1431) << "<- PluginClass::NewInstance = 0" << endl;
       return DCOPRef();
    }
 
-   // display plugin
-   inst->setWindow();
-
    // create source stream
-   if ( !src.isEmpty() ) inst->requestURL( src, mimeType, QString::null, 0 );
+   if ( !src.isEmpty() )
+      inst->requestURL( src, mimeType, QString::null, 0 );
 
-   kdDebug(1431) << "<- NSPluginClass::NewInstance = " << (void*)inst << endl;
    _instances.append( inst );
    return DCOPRef(kapp->dcopClient()->appId(), inst->objId());
 }
@@ -970,7 +1339,7 @@ void NSPluginClass::destroyInstance( NSPluginInstance* inst )
 {
     // mark for destruction
     _trash.append( inst );
-    _timer->start( 0, TRUE );
+    timer(); //_timer->start( 0, TRUE );
 }
 
 
@@ -987,11 +1356,16 @@ NSPluginStreamBase::~NSPluginStreamBase()
 {
    if (_stream) {
       _instance->NPDestroyStream( _stream, NPRES_USER_BREAK );
+      if (_stream && _stream->url)
+          free(const_cast<char*>(_stream->url));
       delete _stream;
+      _stream = 0;
    }
 
    delete _tempFile;
+   _tempFile = 0;
    delete _queue;
+   _queue = 0;
 }
 
 
@@ -1044,7 +1418,7 @@ void NSPluginStreamBase::inform()
 
 }
 
-bool NSPluginStreamBase::create( QString url, QString mimeType, void *notify )
+bool NSPluginStreamBase::create( const QString& url, const QString& mimeType, void *notify )
 {
     if ( _stream )
         return false;
@@ -1066,6 +1440,8 @@ bool NSPluginStreamBase::create( QString url, QString mimeType, void *notify )
     _stream->lastmodified = 0;
     _stream->notifyData = _notifyData;
 
+    _mimeType = mimeType;
+
     return true;
 }
 
@@ -1111,7 +1487,7 @@ int NSPluginStreamBase::process( const QByteArray &data, int start )
 
 bool NSPluginStreamBase::pump()
 {
-    kdDebug(1431) << "queue pos " << _queuePos << ", size " << _queue.size() << endl;
+    //kdDebug(1431) << "queue pos " << _queuePos << ", size " << _queue.size() << endl;
 
     inform();
 
@@ -1144,15 +1520,15 @@ bool NSPluginStreamBase::pump()
 
 void NSPluginStreamBase::queue( const QByteArray &data )
 {
-    kdDebug(1431) << "new queue size " << data.size() << endl;
-    kdDebug(1431) << "data=" << (void*)data.data() << " size=" << data.size() << endl;
-
     _queue = data;
     _queue.detach();
     _queuePos = 0;
     _tries = 0;
 
-    kdDebug(1431) << "queue=" << (void*)_queue.data() << " size=" << _queue.size() << endl;
+    kdDebug(1431) << "new queue size=" << data.size()
+                  << " data=" << (void*)data.data() 
+                  << " queue=" << (void*)_queue.data() << " qsize="
+                  << _queue.size() << endl;
 }
 
 
@@ -1193,6 +1569,8 @@ void NSPluginStreamBase::finish( bool err )
     }
 
     // delete stream
+    if (_stream && _stream->url)
+        free(const_cast<char *>(_stream->url));
     delete _stream;
     _stream = 0;
 
@@ -1217,8 +1595,9 @@ NSPluginBufStream::~NSPluginBufStream()
 }
 
 
-bool NSPluginBufStream::get( QString url, QString mimeType, const QByteArray &buf,
-                             void *notifyData, bool singleShot )
+bool NSPluginBufStream::get( const QString& url, const QString& mimeType,
+                             const QByteArray &buf, void *notifyData,
+                             bool singleShot )
 {
     _singleShot = singleShot;
     if ( create( url, mimeType, notifyData ) ) {
@@ -1263,17 +1642,43 @@ NSPluginStream::~NSPluginStream()
 }
 
 
-bool NSPluginStream::get( QString url, QString mimeType, void *notify )
+bool NSPluginStream::get( const QString& url, const QString& mimeType,
+                          void *notify )
 {
     // create new stream
     if ( create( url, mimeType, notify ) ) {
         // start the kio job
         _job = KIO::get(url, false, false);
+        _job->addMetaData("errorPage", "false");
+        _job->addMetaData("AllowCompressedPage", "false");
         connect(_job, SIGNAL(data(KIO::Job *, const QByteArray &)),
                 SLOT(data(KIO::Job *, const QByteArray &)));
         connect(_job, SIGNAL(result(KIO::Job *)), SLOT(result(KIO::Job *)));
-        connect(_job, SIGNAL(totalSize(KIO::Job *, unsigned long )),
-                SLOT(totalSize(KIO::Job *, unsigned long)));
+        connect(_job, SIGNAL(totalSize(KIO::Job *, KIO::filesize_t )),
+                SLOT(totalSize(KIO::Job *, KIO::filesize_t)));
+        connect(_job, SIGNAL(mimetype(KIO::Job *, const QString &)),
+                SLOT(mimetype(KIO::Job *, const QString &)));
+    }
+
+    return false;
+}
+
+
+bool NSPluginStream::post( const QString& url, const QByteArray& data, 
+           const QString& mimeType, void *notify, const KParts::URLArgs& args )
+{
+    // create new stream
+    if ( create( url, mimeType, notify ) ) {
+        // start the kio job
+        _job = KIO::http_post(url, data, false);
+        _job->addMetaData("content-type", args.contentType());
+        _job->addMetaData("errorPage", "false");
+        _job->addMetaData("AllowCompressedPage", "false");
+        connect(_job, SIGNAL(data(KIO::Job *, const QByteArray &)),
+                SLOT(data(KIO::Job *, const QByteArray &)));
+        connect(_job, SIGNAL(result(KIO::Job *)), SLOT(result(KIO::Job *)));
+        connect(_job, SIGNAL(totalSize(KIO::Job *, KIO::filesize_t )),
+                SLOT(totalSize(KIO::Job *, KIO::filesize_t)));
         connect(_job, SIGNAL(mimetype(KIO::Job *, const QString &)),
                 SLOT(mimetype(KIO::Job *, const QString &)));
     }
@@ -1292,9 +1697,9 @@ void NSPluginStream::data(KIO::Job * job, const QByteArray &data)
     }
 }
 
-void NSPluginStream::totalSize(KIO::Job * job, unsigned long size)
+void NSPluginStream::totalSize(KIO::Job * job, KIO::filesize_t size)
 {
-    kdDebug(1431) << "NSPluginStream::totalSize - job=" << (void*)job << " size=" << size << endl;
+    kdDebug(1431) << "NSPluginStream::totalSize - job=" << (void*)job << " size=" << KIO::number(size) << endl;
     _stream->end = size;
 }
 
